;Git Developer Pack installation for Windows
;Written by Khaled El Mansoury
;http://www.expressprogs.com

;--------------------------------
;Include Modern UI

	!include "MUI2.nsh"
	!include "nsisXML.nsh"
	!include "LogicLib.nsh"
	!include "Timestamp.nsh"
	!include "GetRoot.nsh"
	!include "WriteToFile.nsh"
	!include "EnvVarUpdate.nsh"
	!include "TextReplace.nsh"
	!include "x64.nsh"
	!include "GetParameters.nsh"
	!include "GetParameterValue.nsh"

;--------------------------------
;General

	;Name and file
	Name "Git Developer Pack"
	OutFile "GitDeveloper.exe"
	ShowInstDetails show
	;Default installation folder
	InstallDir "$TEMP\gitdeveloper"

	;Request application privileges for Windows Vista/7
	RequestExecutionLevel admin
	
	;For PATH variable updates
	!define env_hklm 'HKLM "SYSTEM\CurrentControlSet\Control\Session Manager\Environment"'
	!define env_hkcu 'HKCU "Environment"'

;--------------------------------
;Interface Settings

	!define MUI_ABORTWARNING

;--------------------------------
;Pages

	!insertmacro MUI_PAGE_WELCOME
	!insertmacro MUI_PAGE_LICENSE "LICENSE.txt"
	!insertmacro MUI_PAGE_COMPONENTS
	!insertmacro MUI_PAGE_INSTFILES
	!insertmacro MUI_PAGE_FINISH

;--------------------------------
;Languages

	!insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections
Section # "Prerequisites (required)" prerequisite
	;Get Timestamp
	${TimeStamp} $R9
	SetOutPath "$INSTDIR"

	;Download required info.xml and dllist.xml files
	
	${If} ${RunningX64}
	NSISdl::download_quiet http://system.expressprogs.com/gitdeveloperpack/x64/info.xml?$R9 info.xml
	${Else}
	NSISdl::download_quiet http://system.expressprogs.com/gitdeveloperpack/x86/info.xml?$R9 info.xml
	${EndIf}

	${If} ${RunningX64}
	NSISdl::download_quiet http://system.expressprogs.com/gitdeveloperpack/x64/dllist.xml?$R9 dllist.xml
	${Else}
	NSISdl::download_quiet http://system.expressprogs.com/gitdeveloperpack/x86/dllist.xml?$R9 dllist.xml
	${EndIf}
	
	;Check if installer needs to be redownloaded.
	${nsisXML->xPath} "info.xml" "/items/item[@name='installerver']" ${VAR_1}
	${If} $1 != "1.0.0"
	MessageBox MB_OK|MB_ICONSTOP 'This installer is out of date, please re-download from expressprogs.com (Current version: $1, This version: 1.0.0).'
	Quit
	${EndIf}

SectionEnd

Section "msysgit (required)" Git

	;Getting required information to download
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='msysgit']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='msysgit']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='msysgit']/@sha1" ${VAR_4}
	DetailPrint "Downloading msysgit..."
	NSISdl::download $1 "msysgit.exe"
	
	;Checking if download failed
	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	;Getting system drive
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	;Writing install bat file
	Push `cd $INSTDIR $\r$\n msysgit.exe /silent`
	Push `install-msysgit.bat`
	Call WriteToFile
	DetailPrint "Installing msysgit..."
	
	;Running the install bat file
	ExecDos::exec 'install-msysgit.bat' "" "msysgit-log.txt"
	
	;Updating the PATH environment variable
	${EnvVarUpdate} $R8 "Path" "A" "HKLM" "$PROGRAMFILES32\Git\cmd"
	
SectionEnd

Section "PuTTY (required)" PuTTY

	${nsisXML->xPath} "dllist.xml" "/items/item[@name='putty']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='putty']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='putty']/@md5" ${VAR_3}
	DetailPrint "Downloading putty..."
	NSISdl::download $1 "putty.exe"

	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	Push `cd $INSTDIR $\r$\n putty.exe /silent`
	Push `install-putty.bat`
	Call WriteToFile
	DetailPrint "Installing PuTTY..."
	
	ExecDos::exec 'install-putty.bat' "" "putty-log.txt"
	
	${EnvVarUpdate} $R8 "Path" "A" "HKLM" "$PROGRAMFILES32\PuTTY"
	
SectionEnd

Section "Python (required)" Python

	${nsisXML->xPath} "dllist.xml" "/items/item[@name='python']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='python']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='python']/@md5" ${VAR_3}
	DetailPrint "Downloading python..."
	NSISdl::download $1 "python.msi"

	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	DetailPrint "Installing Python..."
	ExecWait '"msiexec" /i "python.msi" /passive' $R7
	
	${nsisXML->xPath} "info.xml" "/items/item[@name='pythonfolder']" ${VAR_1}
	
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	${EnvVarUpdate} $R8 "Path" "A" "HKLM" "$0\$1"
	${EnvVarUpdate} $R8 "Path" "A" "HKLM" "$0\$1\Scripts"
SectionEnd

Section "setuptools (required)" setuptools

	${nsisXML->xPath} "dllist.xml" "/items/item[@name='setuptools']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='setuptools']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='setuptools']/@md5" ${VAR_3}
	DetailPrint "Downloading setuptools..."
	${If} ${RunningX64}
	NSISdl::download $1 "ez_setup.py"

	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	;Hack to make below work (needs DetailPrint)
	DetailPrint ""
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	${nsisXML->xPath} "info.xml" "/items/item[@name='pythonfolder']" ${VAR_1}
	
	Push `SET PATH=%PATH%;$0\$1$\r$\ncd $INSTDIR $\r$\n python.exe ez_setup.py`
	Push `install-setuptools.bat`
	Call WriteToFile
	
	DetailPrint "Installing setuptools..."
	ExecDos::exec 'install-setuptools.bat' "" "setuptools-log.txt"

	${Else}
	NSISdl::download $1 "setuptools.exe"

	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	DetailPrint "Installing setuptools..."
	ExecWait '"setuptools.exe"' $R7
	
	${EndIf}
SectionEnd

Section "pip (required)" pip

	${nsisXML->xPath} "dllist.xml" "/items/item[@name='pip']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='pip']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='pip']/@md5" ${VAR_3}
	DetailPrint "Downloading pip..."
	NSISdl::download $1 "pip.tar.gz"
	
	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	DetailPrint "Extracting pip..."
	untgz::extract -z pip.tar.gz
	DetailPrint "Extracting pip..."
	${nsisXML->xPath} "info.xml" "/items/item[@name='pythonfolder']" ${VAR_1}
	${nsisXML->xPath} "info.xml" "/items/item[@name='pipfolder']" ${VAR_2}
	
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	Push `SET PATH=%PATH%;$0\$1$\r$\ncd $INSTDIR\$2 $\r$\n python.exe setup.py install$\r$\n`
	Push `install-pip.bat`
	Call WriteToFile
	DetailPrint "Installing pip..."
	ExecDos::exec 'install-pip.bat' "" "pip-log.txt"
	
SectionEnd

Section "git-review (required)" git-review

	
	Push `SET PATH=%PATH%;$0\$1;$0\$1\Scripts$\r$\ncd $INSTDIR\$2 $\r$\npip.exe install git-review$\r$\n`
	Push `install-greview.bat`
	Call WriteToFile
	
	Push `@python $0\$1\Scripts\git-review %*`
	Push `$0\$1\git-review.bat`
	Call WriteToFile
	DetailPrint "Installing git-review..."
	ExecDos::exec 'install-greview.bat' "" "greview-log.txt"
SectionEnd

Section "Patch git-review" patch-git-review

	${nsisXML->xPath} "info.xml" "/items/item[@name='git-review-search']" ${VAR_3}
	${nsisXML->xPath} "info.xml" "/items/item[@name='git-review-replace']" ${VAR_4}
	
	${nsisXML->xPath} "info.xml" "/items/item[@name='pythonfolder']" ${VAR_1}
	
	Push "$TEMP"
	Call GetRoot
	Pop $0
	
	DetailPrint "Patching git-review..."
	
	${textreplace::ReplaceInFile} "$0\$1\Scripts\git-review" "$0\$1\Scripts\git-review" "$3" "$4" "/S=1 /C=1" $R5
	${textreplace::Unload}
	
SectionEnd

Section "TortoiseGit" TortoiseGit
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='tortoisegit']" ${VAR_1}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='tortoisegit']/@version" ${VAR_2}
	${nsisXML->xPath} "dllist.xml" "/items/item[@name='tortoisegit']/@sha1" ${VAR_3}
	DetailPrint "Downloading tortoisegit..."
	NSISdl::download $1 "tortoisegit.msi"
	
	Pop $R0 ;Get the return value
	StrCmp $R0 "success" +3
	MessageBox MB_OK "Download failed: $R0"
	
	DetailPrint "Installing TortoiseGit..."
	ExecWait '"msiexec" /i "tortoisegit.msi" /passive' $R7
SectionEnd

Section #"Clean up" Cleanup
	SetOutPath $TEMP
	;Setting prompt for Reboot
	SetRebootFlag true
	
	;Delete temp directory if /debug was not given
	Push "debug"
  Push "0" ;default value
  Call GetParameterValue
  Pop $2
  
  ${If} $2 == "0"
	RMDir /r /REBOOTOK $INSTDIR
	${EndIf}
SectionEnd

LangString DESC_Git ${LANG_ENGLISH} "Git for Windows, main program."
LangString DESC_PuTTY ${LANG_ENGLISH} "Connection tools."
LangString DESC_Python ${LANG_ENGLISH} "Required to install setuptools, pip and git-review."
LangString DESC_setuptools ${LANG_ENGLISH} "Required to install pip and git-review, depends on Python."
LangString DESC_pip ${LANG_ENGLISH} "Required to install git-review, depends on setuptools and Python."
LangString DESC_git-review ${LANG_ENGLISH} "Depends on msysgit, Python, setuptools and pip."
LangString DESC_patch-git-review ${LANG_ENGLISH} "Patches the git-review file in accordance with MediaWiki development in order to download the commit-msg hook. Depends on msysgit, Python, setuptools, pip and get-review."
LangString DESC_TortoiseGit ${LANG_ENGLISH} "Provides a GUI to work with Git. Depends on msysgit."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${Git} $(DESC_Git)
  !insertmacro MUI_DESCRIPTION_TEXT ${PuTTY} $(DESC_PuTTY)
  !insertmacro MUI_DESCRIPTION_TEXT ${Python} $(DESC_Python)
  !insertmacro MUI_DESCRIPTION_TEXT ${setuptools} $(DESC_setuptools)
  !insertmacro MUI_DESCRIPTION_TEXT ${pip} $(DESC_pip)
  !insertmacro MUI_DESCRIPTION_TEXT ${git-review} $(DESC_git-review)
  !insertmacro MUI_DESCRIPTION_TEXT ${patch-git-review} $(DESC_patch-git-review)
  !insertmacro MUI_DESCRIPTION_TEXT ${TortoiseGit} $(DESC_TortoiseGit)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------